@@ -1,6 +1,5 @@
 import logging
 import os
-import types
 from urllib.parse import urlparse
 
 from moto.s3 import models as s3_models
@@ -16,6 +15,7 @@ from localstack.services.s3 import s3_listener, s3_utils
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import get_free_tcp_port, wait_for_port_open
 from localstack.utils.generic.dict_utils import get_safe
+from localstack.utils.patch import patch
 from localstack.utils.server import multiserver
 
 LOG = logging.getLogger(__name__)
@@ -51,7 +51,7 @@ def check_s3(expect_shutdown=False, print_error=False):
 
 
 def start_s3(port=None, backend_port=None, asynchronous=None, update_listener=None):
-    port = port or config.PORT_S3
+    port = port or config.service_port("s3")
     if not backend_port:
         if config.FORWARD_EDGE_INMEM:
             backend_port = multiserver.get_moto_server_port()
@@ -97,51 +97,30 @@ def apply_patches():
         if acl:
             key.set_acl(acl)
 
-    # patch Bucket.create_from_cloudformation_json in moto
-    @classmethod
-    def Bucket_create_from_cloudformation_json(
-        cls, resource_name, cloudformation_json, region_name
-    ):
-        result = create_from_cloudformation_json_orig(
-            resource_name, cloudformation_json, region_name
-        )
-        # remove the bucket from the backend, as our template_deployer will take care of creating the resource
-        resource_name = s3_listener.normalize_bucket_name(resource_name)
-        s3_models.s3_backend.buckets.pop(resource_name)
-        return result
-
-    create_from_cloudformation_json_orig = s3_models.FakeBucket.create_from_cloudformation_json
-    s3_models.FakeBucket.create_from_cloudformation_json = Bucket_create_from_cloudformation_json
-
     # patch S3Bucket.create_bucket(..)
-    def create_bucket(self, bucket_name, region_name, *args, **kwargs):
+    @patch(s3_models.s3_backend.create_bucket)
+    def create_bucket(self, fn, bucket_name, region_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
-        return create_bucket_orig(bucket_name, region_name, *args, **kwargs)
-
-    create_bucket_orig = s3_models.s3_backend.create_bucket
-    s3_models.s3_backend.create_bucket = types.MethodType(create_bucket, s3_models.s3_backend)
+        return fn(bucket_name, region_name, *args, **kwargs)
 
     # patch S3Bucket.get_bucket(..)
-    def get_bucket(self, bucket_name, *args, **kwargs):
+    @patch(s3_models.s3_backend.get_bucket)
+    def get_bucket(self, fn, bucket_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
         if bucket_name == config.BUCKET_MARKER_LOCAL:
             return None
-        return get_bucket_orig(bucket_name, *args, **kwargs)
+        return fn(bucket_name, *args, **kwargs)
 
-    get_bucket_orig = s3_models.s3_backend.get_bucket
-    s3_models.s3_backend.get_bucket = types.MethodType(get_bucket, s3_models.s3_backend)
-
-    def _bucket_response_head(self, bucket_name, *args, **kwargs):
-        code, headers, body = _bucket_response_head_orig(self, bucket_name, *args, **kwargs)
+    @patch(s3_responses.ResponseObject._bucket_response_head)
+    def _bucket_response_head(fn, self, bucket_name, *args, **kwargs):
+        code, headers, body = fn(self, bucket_name, *args, **kwargs)
         bucket = s3_models.s3_backend.get_bucket(bucket_name)
         headers["x-amz-bucket-region"] = bucket.region_name
         return code, headers, body
 
-    _bucket_response_head_orig = s3_responses.ResponseObject._bucket_response_head
-    s3_responses.ResponseObject._bucket_response_head = _bucket_response_head
-
-    def _bucket_response_get(self, bucket_name, querystring, *args, **kwargs):
-        result = _bucket_response_get_orig(self, bucket_name, querystring, *args, **kwargs)
+    @patch(s3_responses.ResponseObject._bucket_response_get)
+    def _bucket_response_get(fn, self, bucket_name, querystring, *args, **kwargs):
+        result = fn(self, bucket_name, querystring, *args, **kwargs)
         # for some reason in the "get-bucket-location" call, moto doesn't return a code, headers, body triple as a result
         if isinstance(result, tuple) and len(result) == 3:
             code, headers, body = result
@@ -149,24 +128,19 @@ def apply_patches():
             headers["x-amz-bucket-region"] = bucket.region_name
         return result
 
-    _bucket_response_get_orig = s3_responses.ResponseObject._bucket_response_get
-    # note: method type is only required when the method is part of an object instance (make a method out of a function)
-    s3_responses.ResponseObject._bucket_response_get = _bucket_response_get
-
     # patch S3Bucket.get_bucket(..)
-    def delete_bucket(self, bucket_name, *args, **kwargs):
+    @patch(s3_models.s3_backend.delete_bucket)
+    def delete_bucket(self, fn, bucket_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
         s3_listener.remove_bucket_notification(bucket_name)
-        return delete_bucket_orig(bucket_name, *args, **kwargs)
-
-    delete_bucket_orig = s3_models.s3_backend.delete_bucket
-    s3_models.s3_backend.delete_bucket = types.MethodType(delete_bucket, s3_models.s3_backend)
+        return fn(bucket_name, *args, **kwargs)
 
     # patch _key_response_post(..)
-    def s3_key_response_post(self, request, body, bucket_name, query, key_name, *args, **kwargs):
-        result = s3_key_response_post_orig(
-            request, body, bucket_name, query, key_name, *args, **kwargs
-        )
+    @patch(s3_responses.S3ResponseInstance._key_response_post)
+    def s3_key_response_post(
+        self, fn, request, body, bucket_name, query, key_name, *args, **kwargs
+    ):
+        result = fn(request, body, bucket_name, query, key_name, *args, **kwargs)
         s3_update_acls(self, request, query, bucket_name, key_name)
         try:
             if query.get("uploadId"):
@@ -183,28 +157,18 @@ def apply_patches():
             TMP_TAG[(bucket_name, key_name)] = tags
         return result
 
-    s3_key_response_post_orig = s3_responses.S3ResponseInstance._key_response_post
-    s3_responses.S3ResponseInstance._key_response_post = types.MethodType(
-        s3_key_response_post, s3_responses.S3ResponseInstance
-    )
-
     # patch _key_response_put(..)
+    @patch(s3_responses.S3ResponseInstance._key_response_put)
     def s3_key_response_put(
-        self, request, body, bucket_name, query, key_name, headers, *args, **kwargs
+        self, fn, request, body, bucket_name, query, key_name, headers, *args, **kwargs
     ):
-        result = s3_key_response_put_orig(
-            request, body, bucket_name, query, key_name, headers, *args, **kwargs
-        )
+        result = fn(request, body, bucket_name, query, key_name, headers, *args, **kwargs)
         s3_update_acls(self, request, query, bucket_name, key_name)
         return result
 
-    s3_key_response_put_orig = s3_responses.S3ResponseInstance._key_response_put
-    s3_responses.S3ResponseInstance._key_response_put = types.MethodType(
-        s3_key_response_put, s3_responses.S3ResponseInstance
-    )
-
     # patch DeleteObjectTagging
-    def s3_key_response_delete(self, headers, bucket_name, query, key_name, *args, **kwargs):
+    @patch(s3_responses.S3ResponseInstance._key_response_delete)
+    def s3_key_response_delete(self, fn, headers, bucket_name, query, key_name, *args, **kwargs):
         # Fixes https://github.com/localstack/localstack/issues/1083
         if query.get("tagging"):
             self._set_action("KEY", "DELETE", query)
@@ -213,13 +177,9 @@ def apply_patches():
             key.tags = {}
             self.backend.tagger.delete_all_tags_for_resource(key.arn)
             return 204, {}, ""
-        result = s3_key_response_delete_orig(headers, bucket_name, query, key_name, *args, **kwargs)
+        result = fn(headers, bucket_name, query, key_name, *args, **kwargs)
         return result
 
-    s3_key_response_delete_orig = s3_responses.S3ResponseInstance._key_response_delete
-    s3_responses.S3ResponseInstance._key_response_delete = types.MethodType(
-        s3_key_response_delete, s3_responses.S3ResponseInstance
-    )
     action_map = s3_responses.ACTION_MAP
     action_map["KEY"]["DELETE"]["tagging"] = (
         action_map["KEY"]["DELETE"].get("tagging") or "DeleteObjectTagging"
@@ -238,8 +198,9 @@ def apply_patches():
                 **kwargs,
             )
 
-    def s3_key_response_get(self, bucket_name, query, key_name, headers, *args, **kwargs):
-        resp_status, resp_headers, resp_value = s3_key_response_get_orig(
+    @patch(s3_responses.S3ResponseInstance._key_response_get)
+    def s3_key_response_get(self, fn, bucket_name, query, key_name, headers, *args, **kwargs):
+        resp_status, resp_headers, resp_value = fn(
             bucket_name, query, key_name, headers, *args, **kwargs
         )
 
@@ -250,22 +211,14 @@ def apply_patches():
 
         return resp_status, resp_headers, resp_value
 
-    s3_key_response_get_orig = s3_responses.S3ResponseInstance._key_response_get
-    s3_responses.S3ResponseInstance._key_response_get = types.MethodType(
-        s3_key_response_get, s3_responses.S3ResponseInstance
-    )
-
-    # patch max-keys
-    def s3_truncate_result(self, result_keys, max_keys):
-        return s3_truncate_result_orig(result_keys, max_keys or 1000)
-
-    s3_truncate_result_orig = s3_responses.S3ResponseInstance._truncate_result
-    s3_responses.S3ResponseInstance._truncate_result = types.MethodType(
-        s3_truncate_result, s3_responses.S3ResponseInstance
-    )
+    # patch truncate_result
+    @patch(s3_responses.S3ResponseInstance._truncate_result)
+    def s3_truncate_result(self, fn, result_keys, max_keys):
+        return fn(result_keys, max_keys or 1000)
 
     # patch _bucket_response_delete_keys(..)
     # https://github.com/localstack/localstack/issues/2077
+    # TODO: check if patch still needed!
     s3_delete_keys_response_template = """<?xml version="1.0" encoding="UTF-8"?>
     <DeleteResult xmlns="http://s3.amazonaws.com/doc/2006-03-01">
     {% for k in deleted %}
@@ -281,6 +234,7 @@ def apply_patches():
     {% endfor %}
     </DeleteResult>"""
 
+    @patch(s3_responses.S3ResponseInstance._bucket_response_delete_keys, pass_target=False)
     def s3_bucket_response_delete_keys(self, request, body, bucket_name):
         template = self.response_template(s3_delete_keys_response_template)
         elements = minidom.parseString(body).getElementsByTagName("Object")
@@ -322,18 +276,12 @@ def apply_patches():
             template.render(deleted=deleted_names, delete_errors=error_names),
         )
 
-    s3_responses.S3ResponseInstance._bucket_response_delete_keys = types.MethodType(
-        s3_bucket_response_delete_keys, s3_responses.S3ResponseInstance
-    )
-
     # Patch _handle_range_header(..)
     # https://github.com/localstack/localstack/issues/2146
-    s3_response_handle_range_header_orig = s3_responses.S3ResponseInstance._handle_range_header
 
-    def s3_response_handle_range_header(self, request, headers, response_content):
-        rs_code, rs_headers, rs_content = s3_response_handle_range_header_orig(
-            request, headers, response_content
-        )
+    @patch(s3_responses.S3ResponseInstance._handle_range_header)
+    def s3_response_handle_range_header(self, fn, request, headers, response_content):
+        rs_code, rs_headers, rs_content = fn(request, headers, response_content)
         if rs_code == 206:
             for k in ["ETag", "last-modified"]:
                 v = headers.get(k)
@@ -342,10 +290,6 @@ def apply_patches():
 
         return rs_code, rs_headers, rs_content
 
-    s3_responses.S3ResponseInstance._handle_range_header = types.MethodType(
-        s3_response_handle_range_header, s3_responses.S3ResponseInstance
-    )
-
     # Patch utils_is_delete_keys
     # https://github.com/localstack/localstack/issues/2866
     # https://github.com/localstack/localstack/issues/2850
@@ -358,6 +302,7 @@ def apply_patches():
             request, path, bucket_name
         )
 
+    @patch(s3_responses.S3ResponseInstance.is_delete_keys, pass_target=False)
     def s3_response_is_delete_keys(self, request, path, bucket_name):
         if self.subdomain_based_buckets(request):
             # Temporary fix until moto supports x-id and DeleteObjects (#3931)
@@ -369,30 +314,19 @@ def apply_patches():
         else:
             return utils_is_delete_keys(request, path, bucket_name)
 
-    s3_responses.S3ResponseInstance.is_delete_keys = types.MethodType(
-        s3_response_is_delete_keys, s3_responses.S3ResponseInstance
-    )
-
+    @patch(s3_responses.S3ResponseInstance.parse_bucket_name_from_url, pass_target=False)
     def parse_bucket_name_from_url(self, request, url):
         path = urlparse(url).path
         return s3_utils.extract_bucket_name(request.headers, path)
 
-    s3_responses.S3ResponseInstance.parse_bucket_name_from_url = types.MethodType(
-        parse_bucket_name_from_url, s3_responses.S3ResponseInstance
-    )
-
+    @patch(s3_responses.S3ResponseInstance.subdomain_based_buckets, pass_target=False)
     def subdomain_based_buckets(self, request):
         return s3_utils.uses_host_addressing(request.headers)
 
-    s3_responses.S3ResponseInstance.subdomain_based_buckets = types.MethodType(
-        subdomain_based_buckets, s3_responses.S3ResponseInstance
-    )
-
-    s3_responses_bucket_response_get_orig = s3_responses.S3ResponseInstance._bucket_response_get
-
-    def s3_bucket_response_get(self, bucket_name, querystring):
+    @patch(s3_responses.S3ResponseInstance._bucket_response_get)
+    def s3_bucket_response_get(self, fn, bucket_name, querystring):
         try:
-            return s3_responses_bucket_response_get_orig(bucket_name, querystring)
+            return fn(bucket_name, querystring)
         except NotImplementedError:
             if "uploads" not in querystring:
                 raise
@@ -409,37 +343,23 @@ def apply_patches():
             template = self.response_template(S3_ALL_MULTIPARTS)
             return template.render(bucket_name=bucket_name, uploads=multiparts)
 
-    s3_responses.S3ResponseInstance._bucket_response_get = types.MethodType(
-        s3_bucket_response_get, s3_responses.S3ResponseInstance
-    )
-
-    copy_object_orig = s3_models.s3_backend.copy_object
-
+    @patch(s3_models.s3_backend.copy_object)
     def copy_object(
         self,
-        src_bucket_name,
-        src_key_name,
+        fn,
+        src_key,
         dest_bucket_name,
         dest_key_name,
-        storage=None,
-        acl=None,
-        src_version_id=None,
         *args,
         **kwargs,
     ):
-        copy_object_orig(
-            src_bucket_name,
-            src_key_name,
+        fn(
+            src_key,
             dest_bucket_name,
             dest_key_name,
-            storage=storage,
-            acl=acl,
-            src_version_id=src_version_id,
             *args,
             **kwargs,
         )
         key = self.get_object(dest_bucket_name, dest_key_name)
         # reset etag
         key._etag = None
-
-    s3_models.s3_backend.copy_object = types.MethodType(copy_object, s3_models.s3_backend)
