@@ -13,7 +13,9 @@ from pathlib import Path
 from typing import Dict, List, Optional, Tuple, Union
 
 from localstack import config
-from localstack.utils.common import TMP_FILES, HashableList, rm_rf, save_file, short_uid
+from localstack.utils.collections import HashableList
+from localstack.utils.files import TMP_FILES, rm_rf, save_file
+from localstack.utils.strings import short_uid
 
 LOG = logging.getLogger(__name__)
 
@@ -288,7 +290,8 @@ class ContainerConfiguration:
 
     stdin: Optional[str] = None
     user: Optional[str] = None
-    cap_add: Optional[str] = None
+    cap_add: Optional[List[str]] = None
+    cap_drop: Optional[List[str]] = None
     network: Optional[str] = None
     dns: Optional[str] = None
     workdir: Optional[str] = None
@@ -566,7 +569,8 @@ class ContainerClient(metaclass=ABCMeta):
         ports: Optional[PortMappings] = None,
         env_vars: Optional[Dict[str, str]] = None,
         user: Optional[str] = None,
-        cap_add: Optional[str] = None,
+        cap_add: Optional[List[str]] = None,
+        cap_drop: Optional[List[str]] = None,
         network: Optional[str] = None,
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
@@ -595,7 +599,8 @@ class ContainerClient(metaclass=ABCMeta):
         ports: Optional[PortMappings] = None,
         env_vars: Optional[Dict[str, str]] = None,
         user: Optional[str] = None,
-        cap_add: Optional[str] = None,
+        cap_add: Optional[List[str]] = None,
+        cap_drop: Optional[List[str]] = None,
         network: Optional[str] = None,
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
@@ -617,6 +622,7 @@ class ContainerClient(metaclass=ABCMeta):
         env_vars: Optional[Dict[str, Optional[str]]] = None,
         stdin: Optional[bytes] = None,
         user: Optional[str] = None,
+        workdir: Optional[str] = None,
     ) -> Tuple[bytes, bytes]:
         """Execute a given command in a container
 
